"""
播磨町の毎月の人口PDFデータを読み込んでグラフ化するプログラム
"""
#%%
import warnings
warnings.simplefilter('ignore', FutureWarning) #FutureWarningを非表示

#%%
import os
import pandas as pd
import tabula
import matplotlib.pyplot as plt

plt.rcParams['font.family'] = 'ms gothic'
plt.rcParams['xtick.direction'] = 'in'
plt.rcParams['ytick.direction'] = 'in'
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True

#%%

def parse_Harima_population(df, year, month):
    """播磨町ホームページで公開されている毎月の人口PDFを解析する

    ３列に分かれた表を１列に結合し，付加情報として"date"列に年月日情報をいれる

    Args:
        df : PDFから読み取った表のDataFrame
        year: 年
        month : 月
    """
    # 0-34歳
    dfa = df.iloc[:, :4]

    # 35-69歳
    dfb = df.iloc[:, 4:8]
    dfb.columns = dfa.columns

    # 70-105歳
    dfc = df.iloc[:, 8:12]
    dfc.columns = dfa.columns

    #連結し，1行すべてNaNの行を削除
    dfall = pd.concat([dfa,dfb,dfc]).reset_index(drop=True).dropna(how='all')

    #総計を表示
    print(dfall[dfall['年齢']=='総計'])

    #小計・総計の行を削除
    dfall = dfall[dfall['年齢']!='小計']
    dfall = dfall[dfall['年齢']!='総計'].reset_index(drop=True)
    dfall = dfall.fillna(0)

    dfall['年齢'] = dfall['年齢'].str.replace('以上','')
    dfall['年齢'] = dfall['年齢'].str.replace('歳','').astype(int)

    dfall['男性人数'] = dfall['男性人数'].astype('int')
    dfall['女性人数'] = dfall['女性人数'].astype('int')
    dfall['合計'] = dfall['合計'].astype('int')

    print(dfall[['男性人数','女性人数','合計']].sum())

    dfall['date'] = pd.to_datetime(f"{year+2018}{month:02d}01", format='%Y%m%d')

    return dfall


def load_all_pdfs(base_path):
    """ダウンロードした月別年齢別播磨町人口PDFファイルをすべて読み込む

    Args:
        base_path: PDFファイルを保存したフォルダ
    """
    df_all = []
    for year in range(2, 7): #R2~R6
        for month in range(1, 13): # 1月から12月
            pdf_path = os.path.join(base_path, f"{year:02d}{month:02d}.pdf")
            if os.path.exists(pdf_path):
                print(pdf_path)
                dfs = tabula.read_pdf(pdf_path, stream=True, encoding="shift-jis", lattice=True, pages=[1])

                df_all.append(parse_Harima_population(dfs[0], year, month))
    
    return pd.concat(df_all).reset_index(drop=True)

#%%
if __name__ == '__main__':

    base_dir = r"./PdfData"

    # R2 4月分で試す．

    pdf_file = "0204.pdf"
    dfs = tabula.read_pdf(os.path.join(base_dir, pdf_file), stream=True, encoding="shift-jis", lattice=True, pages=[1])
    if len(dfs) == 0:
        print("cannot find table.")
        exit()

    df0204 = parse_Harima_population(dfs[0], 2, 4)
    print(df0204)

    plt.plot(df0204['年齢'], df0204['合計'])
    plt.xlabel('年齢')
    plt.ylabel('人数')
    plt.show()
    #%%

    df_all = load_all_pdfs(base_dir)
    print(df_all)

    #%%
    newborns = df_all[df_all['年齢']==0][['合計','date']]
    newborns.set_index('date').plot()
    plt.show()

    #%%
    df_o80 = df_all[df_all['年齢']>=80]
    df_o80.groupby('date')['合計'].sum().plot()
    plt.title('80歳以上人口')
    plt.ylim(0,3500)
    plt.show()

# %%
