"""
播磨町の毎月の人口PDFデータを読み込んでグラフ化するプログラム
"""

#%%
import os
import numpy as np
import pandas as pd
import tabula
import matplotlib.pyplot as plt

plt.rcParams['font.family'] = 'ms gothic'
plt.rcParams['xtick.direction'] = 'in'
plt.rcParams['ytick.direction'] = 'in'
plt.rcParams['xtick.top'] = True
plt.rcParams['ytick.right'] = True

def parse_Harima_population(df, year, month):
    """播磨町ホームページで公開されている毎月の人口PDFを解析する

    ３列に分かれた表を１列に結合し，付加情報として"date"列に年月日情報をいれる

    Args:
        df : PDFから読み取った表のDataFrame
        year: 年
        month : 月
    """

    try:
        # データフレームを年齢層ごとに分割
        dfa = df.iloc[:, :4]  # 0-34歳
        dfb = df.iloc[:, 4:8]  # 35-69歳
        dfb.columns = dfa.columns  # カラム名を統一
        dfc = df.iloc[:, 8:12]  # 70-105歳
        dfc.columns = dfa.columns  # カラム名を統一

        # データフレームを連結し、NaNのみの行を削除
        dfall = pd.concat([dfa, dfb, dfc]).reset_index(drop=True).dropna(how='all')
        
        # 小計・総計の行を削除
        dfall = dfall[dfall['年齢'] != '小計']
        dfall = dfall[dfall['年齢'] != '総計'].reset_index(drop=True)
        dfall = dfall.fillna(0)  # NaNを0に置換

        dfall['年齢'] = dfall['年齢'].str.replace('以上','')
        dfall['年齢'] = dfall['年齢'].str.replace('歳','').astype(int)
        
        # データ型を整数に変換
        dfall['男性人数'] = dfall['男性人数'].astype('int')
        dfall['女性人数'] = dfall['女性人数'].astype('int')
        dfall['合計'] = dfall['合計'].astype('int')

        # 日付情報を追加
        dfall['date'] = pd.to_datetime(f"{year+2018}{month:02d}01", format='%Y%m%d')

        return dfall  # 加工済みのデータフレームを返す
    except Exception as e:
        print(f"Error processing year: {year}, month: {month}, error: {e}")
        return pd.DataFrame()  # エラー時は空のデータフレームを返す

# 単一のPDFファイルを読み込む関数
def load_pdf_file(pdf_path, year, month):
    try:
        # PDFファイルを読み込み
        dfs = tabula.read_pdf(pdf_path, stream=True, encoding="shift-jis", lattice=True)
        if not dfs:
            print(f"No tables found in {pdf_path}")
            return pd.DataFrame()  # テーブルが見つからない場合は空のデータフレームを返す
        return parse_Harima_population(dfs[0], year, month)  # パース関数を呼び出し
    except Exception as e:
        print(f"Error reading PDF file: {pdf_path}, error: {e}")
        return pd.DataFrame()  # エラー時は空のデータフレームを返す

# すべてのPDFファイルを読み込み、統合する関数
def load_all_pdfs(base_path):
    """ダウンロードした月別年齢別播磨町人口PDFファイルをすべて読み込み結合する

    Args:
        base_path: PDFファイルを保存したフォルダ
    
    Returns:
        df_all : すべての表を１つにまとめたDataFrame
    """

    df_all = []
    for year in range(2, 7):  # 年の範囲（2から6年）
        for month in range(1, 13):  # 月の範囲（1から12月）
            pdf_path = os.path.join(base_path, f"{year:02d}{month:02d}.pdf")
            if os.path.exists(pdf_path):  # PDFファイルが存在するか確認
                print(pdf_path)
                df_all.append(load_pdf_file(pdf_path, year, month))  # データをリストに追加

    # すべてのデータフレームを結合し、インデックスをリセットして返す
    return pd.concat(df_all).reset_index(drop=True) if df_all else pd.DataFrame()

#%%
if __name__ == '__main__':
    base_dir = r"../PdfData"  # ベースディレクトリ

    # すべてのPDFファイルを読み込み、解析結果をプロット
    df_all = load_all_pdfs(base_dir)
    print(df_all)

#%%
    # 0歳人口の推移
    if not df_all.empty:
        newborns = df_all[df_all['年齢'] == 0][['合計', 'date']]
        newborns.set_index('date').plot()
        plt.ylim(0,300)
        plt.title("0歳人口")
        plt.show()

# %%
    # 80歳以上人口の推移
    df_o80 = df_all[df_all['年齢']>=80]
    df_o80.groupby('date')['合計'].sum().plot()
    plt.title('80歳以上人口')
    plt.ylim(0,3500)
    plt.show()

#%%
# %%
    # 播磨町の人口推移
    df_all.groupby('date')['合計'].sum().plot()
    plt.title('播磨町人口')
    plt.ylim(0, 40000)
    plt.show()


#%%
    # 人口ピラミッドを描く
    df_202404 = df_all[df_all['date']=='2024-04-01']
    fig, ax = plt.subplots(ncols=2, figsize=(8,6))

    # 男性人口
    ax[0].barh(df_202404['年齢'], df_202404['男性人数'], color='b', height=0.5, label='男')
    ax[0].yaxis.tick_right() # 軸を右に
    ax[0].set_yticks(np.array(range(0,111,10))) #　10歳刻み
    ax[0].set_yticklabels([]) # こちらの軸ラベルは非表示
    ax[0].set_xlim([400, 0]) # x軸反転
    ax[0].set_ylim(-0.5, 110)
    
    #　女性人口
    ax[1].barh(df_202404['年齢'], df_202404['女性人数'], color='r', height=0.5, label='女')
    ax[1].set_yticks(np.array(range(0,111,10)))
    ax[1].set_xlim([0, 400])
    ax[1].set_xlabel('人口')
    ax[1].set_ylim(-0.5, 110)

    fig.legend(loc='upper right')
    plt.title('播磨町2024年4月人口ピラミッド')
    plt.show()
# %%
