"""
CCS発行の見積書PDFから必要な情報を抽出し，
発注書の雛形Excelファイルを編集する．
"""

#%%
import os
import pandas as pd
import tabula
import datetime
from openpyxl import load_workbook
from pdfminer.high_level import extract_text

def load_pdf_table(pdf_path, encoding="shift-jis"):
    """ 見積書PDFから必要な表を抽出し，整形する

    Args:
        pdf_path : PDFファイルパス
    Return:
        整形した 見積書DataFrame 
    """
    dfs = tabula.read_pdf(pdf_path, stream=True, encoding=encoding, lattice=True, pages=[1])

    dft = pd.DataFrame() # 空のデータフレーム

    # 複数の表を順に調べ
    for df in dfs:
        # 幅７の表がメインの表なので抽出する（この見積書では幅７の表はこれだけ．
        if len(df.columns)==7:
            dft = df.dropna(how='all')

    print("整形前\n", dft)

    if dft.size > 0: #空のデータブレームでなければ（幅７の表が見つかれば）
        # 見出しのスペースを削除し，改行コード(\r)も削除する
        dft.columns = dft.columns.str.replace(' ','')
        # \r に続く任意の文字列を消す（正規表現）
        dft.columns = dft.columns.str.replace('[\r].*', '', regex=True)

        ## 表の中身を整える
        # 商品名称」内の \r を消す
        dft.loc[:, '商品名称'] = dft['商品名称'].str.replace('\r',' ')

        # \r に続く任意の文字列を消す（正規表現）
        dft.loc[:, '商品コード'] = dft['商品コード'].str.replace('[\r].*','', regex=True)
        
        # 文字列に変換して
        dft.loc[:, '個数'] = dft['個数'].astype(str)
        # \r に続く任意の文字列を消す（正規表現）
        dft.loc[:, '個数'] = dft['個数'].str.replace('[\r].*','', regex=True)

        # 桁取りの , を消し，数値に変換する．
        dft.loc[:, '見積単価'] = dft['見積単価'].str.replace(',','').astype('int')

        #標準単価の列は不要なので消す（drop)
        dft = dft.drop('標準単価', axis=1) 

    print("整形後\n", dft)
    return dft


def get_quotation_info(pdf_path):
    """PDF見積書から見積もり番号を抽出する
    """
    text = extract_text(pdf_path)
    text = text.replace(' ','')
    texts = text.split()

    #見積No.を検索し，それに続く文字列を取得する
    qn = texts.index('見積№') 
    quotation_number = texts[qn+2]+texts[qn+3]

    return quotation_number


def make_order_sheet(df, qn_str, base_folder="."):
    """発注書の雛形を用いて，必要事項を見積書から転記する

    Args:
        df:読み取った商品情報のDataFrame
        qn_str:読み取った見積もり番号の文字列
        base_folder:作業フォルダ
    """
    wb = load_workbook(os.path.join(base_folder, r'001_発注書.xlsx'))
    ws = wb.active

    # 見積No.
    ws['N3'].value = qn_str
    
    # 発注日
    dt_now = datetime.datetime.now()
    ws['N4'].value = dt_now.strftime("%Y年%#m月%#d日") #現在の日付

    for index, row in df.iterrows():
        ws[f"A{index+16}"].value = index+1
        ws[f"B{index+16}"].value = f"({row['商品コード']}){row['商品名称']}"
        ws[f"J{index+16}"].value = row['個数']
        ws[f"L{index+16}"].value = row['見積単価']
    
    saveFileName = f'order_{qn_str}.xlsx'
    wb.save(os.path.join(base_folder, saveFileName))
    wb.close()


#%%

if __name__ == '__main__':
    #base_folder = r"Sample2_ex_Automation" #? Run Python File で実行時 
    base_folder = r"." #? Interactive　(ipython notebook) で実行時

    files = os.listdir(base_folder)
    for file in files:
        if file.startswith('御見積書') and file.endswith(".pdf"):
            pdf_path = os.path.join(base_folder, file)
            print(pdf_path)

            qn_str = get_quotation_info(pdf_path)
            print(f"\n見積番号:{qn_str}")

            df = load_pdf_table(pdf_path)

            make_order_sheet(df, qn_str, base_folder)

# %%
